// Copyright 2023 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// GENERATED FROM THE API DEFINITION IN
//   extensions/common/api/web_request.json
// by tools/json_schema_compiler.
// DO NOT EDIT.

#include "extensions/common/api/web_request.h"

#include <memory>
#include <ostream>
#include <string>
#include <utility>
#include <vector>

#include "base/check.h"
#include "base/check_op.h"
#include "base/notreached.h"
#include "base/strings/string_number_conversions.h"
#include "base/strings/utf_string_conversions.h"
#include "base/values.h"
#include "tools/json_schema_compiler/util.h"
#include "extensions/common/api/extension_types.h"


using base::UTF8ToUTF16;

namespace extensions {
namespace api {
namespace web_request {
//
// Properties
//

const int MAX_HANDLER_BEHAVIOR_CHANGED_CALLS_PER_10_MINUTES = 20;

//
// Types
//

const char* ToString(ResourceType enum_param) {
  switch (enum_param) {
    case RESOURCE_TYPE_MAIN_FRAME:
      return "main_frame";
    case RESOURCE_TYPE_SUB_FRAME:
      return "sub_frame";
    case RESOURCE_TYPE_STYLESHEET:
      return "stylesheet";
    case RESOURCE_TYPE_SCRIPT:
      return "script";
    case RESOURCE_TYPE_IMAGE:
      return "image";
    case RESOURCE_TYPE_FONT:
      return "font";
    case RESOURCE_TYPE_OBJECT:
      return "object";
    case RESOURCE_TYPE_XMLHTTPREQUEST:
      return "xmlhttprequest";
    case RESOURCE_TYPE_PING:
      return "ping";
    case RESOURCE_TYPE_CSP_REPORT:
      return "csp_report";
    case RESOURCE_TYPE_MEDIA:
      return "media";
    case RESOURCE_TYPE_WEBSOCKET:
      return "websocket";
    case RESOURCE_TYPE_OTHER:
      return "other";
    case RESOURCE_TYPE_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

ResourceType ParseResourceType(const std::string& enum_string) {
  if (enum_string == "main_frame")
    return RESOURCE_TYPE_MAIN_FRAME;
  if (enum_string == "sub_frame")
    return RESOURCE_TYPE_SUB_FRAME;
  if (enum_string == "stylesheet")
    return RESOURCE_TYPE_STYLESHEET;
  if (enum_string == "script")
    return RESOURCE_TYPE_SCRIPT;
  if (enum_string == "image")
    return RESOURCE_TYPE_IMAGE;
  if (enum_string == "font")
    return RESOURCE_TYPE_FONT;
  if (enum_string == "object")
    return RESOURCE_TYPE_OBJECT;
  if (enum_string == "xmlhttprequest")
    return RESOURCE_TYPE_XMLHTTPREQUEST;
  if (enum_string == "ping")
    return RESOURCE_TYPE_PING;
  if (enum_string == "csp_report")
    return RESOURCE_TYPE_CSP_REPORT;
  if (enum_string == "media")
    return RESOURCE_TYPE_MEDIA;
  if (enum_string == "websocket")
    return RESOURCE_TYPE_WEBSOCKET;
  if (enum_string == "other")
    return RESOURCE_TYPE_OTHER;
  return RESOURCE_TYPE_NONE;
}


const char* ToString(OnBeforeRequestOptions enum_param) {
  switch (enum_param) {
    case ON_BEFORE_REQUEST_OPTIONS_BLOCKING:
      return "blocking";
    case ON_BEFORE_REQUEST_OPTIONS_REQUESTBODY:
      return "requestBody";
    case ON_BEFORE_REQUEST_OPTIONS_EXTRAHEADERS:
      return "extraHeaders";
    case ON_BEFORE_REQUEST_OPTIONS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnBeforeRequestOptions ParseOnBeforeRequestOptions(const std::string& enum_string) {
  if (enum_string == "blocking")
    return ON_BEFORE_REQUEST_OPTIONS_BLOCKING;
  if (enum_string == "requestBody")
    return ON_BEFORE_REQUEST_OPTIONS_REQUESTBODY;
  if (enum_string == "extraHeaders")
    return ON_BEFORE_REQUEST_OPTIONS_EXTRAHEADERS;
  return ON_BEFORE_REQUEST_OPTIONS_NONE;
}


const char* ToString(OnBeforeSendHeadersOptions enum_param) {
  switch (enum_param) {
    case ON_BEFORE_SEND_HEADERS_OPTIONS_REQUESTHEADERS:
      return "requestHeaders";
    case ON_BEFORE_SEND_HEADERS_OPTIONS_BLOCKING:
      return "blocking";
    case ON_BEFORE_SEND_HEADERS_OPTIONS_EXTRAHEADERS:
      return "extraHeaders";
    case ON_BEFORE_SEND_HEADERS_OPTIONS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnBeforeSendHeadersOptions ParseOnBeforeSendHeadersOptions(const std::string& enum_string) {
  if (enum_string == "requestHeaders")
    return ON_BEFORE_SEND_HEADERS_OPTIONS_REQUESTHEADERS;
  if (enum_string == "blocking")
    return ON_BEFORE_SEND_HEADERS_OPTIONS_BLOCKING;
  if (enum_string == "extraHeaders")
    return ON_BEFORE_SEND_HEADERS_OPTIONS_EXTRAHEADERS;
  return ON_BEFORE_SEND_HEADERS_OPTIONS_NONE;
}


const char* ToString(OnSendHeadersOptions enum_param) {
  switch (enum_param) {
    case ON_SEND_HEADERS_OPTIONS_REQUESTHEADERS:
      return "requestHeaders";
    case ON_SEND_HEADERS_OPTIONS_EXTRAHEADERS:
      return "extraHeaders";
    case ON_SEND_HEADERS_OPTIONS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnSendHeadersOptions ParseOnSendHeadersOptions(const std::string& enum_string) {
  if (enum_string == "requestHeaders")
    return ON_SEND_HEADERS_OPTIONS_REQUESTHEADERS;
  if (enum_string == "extraHeaders")
    return ON_SEND_HEADERS_OPTIONS_EXTRAHEADERS;
  return ON_SEND_HEADERS_OPTIONS_NONE;
}


const char* ToString(OnHeadersReceivedOptions enum_param) {
  switch (enum_param) {
    case ON_HEADERS_RECEIVED_OPTIONS_BLOCKING:
      return "blocking";
    case ON_HEADERS_RECEIVED_OPTIONS_RESPONSEHEADERS:
      return "responseHeaders";
    case ON_HEADERS_RECEIVED_OPTIONS_EXTRAHEADERS:
      return "extraHeaders";
    case ON_HEADERS_RECEIVED_OPTIONS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnHeadersReceivedOptions ParseOnHeadersReceivedOptions(const std::string& enum_string) {
  if (enum_string == "blocking")
    return ON_HEADERS_RECEIVED_OPTIONS_BLOCKING;
  if (enum_string == "responseHeaders")
    return ON_HEADERS_RECEIVED_OPTIONS_RESPONSEHEADERS;
  if (enum_string == "extraHeaders")
    return ON_HEADERS_RECEIVED_OPTIONS_EXTRAHEADERS;
  return ON_HEADERS_RECEIVED_OPTIONS_NONE;
}


const char* ToString(OnAuthRequiredOptions enum_param) {
  switch (enum_param) {
    case ON_AUTH_REQUIRED_OPTIONS_RESPONSEHEADERS:
      return "responseHeaders";
    case ON_AUTH_REQUIRED_OPTIONS_BLOCKING:
      return "blocking";
    case ON_AUTH_REQUIRED_OPTIONS_ASYNCBLOCKING:
      return "asyncBlocking";
    case ON_AUTH_REQUIRED_OPTIONS_EXTRAHEADERS:
      return "extraHeaders";
    case ON_AUTH_REQUIRED_OPTIONS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnAuthRequiredOptions ParseOnAuthRequiredOptions(const std::string& enum_string) {
  if (enum_string == "responseHeaders")
    return ON_AUTH_REQUIRED_OPTIONS_RESPONSEHEADERS;
  if (enum_string == "blocking")
    return ON_AUTH_REQUIRED_OPTIONS_BLOCKING;
  if (enum_string == "asyncBlocking")
    return ON_AUTH_REQUIRED_OPTIONS_ASYNCBLOCKING;
  if (enum_string == "extraHeaders")
    return ON_AUTH_REQUIRED_OPTIONS_EXTRAHEADERS;
  return ON_AUTH_REQUIRED_OPTIONS_NONE;
}


const char* ToString(OnResponseStartedOptions enum_param) {
  switch (enum_param) {
    case ON_RESPONSE_STARTED_OPTIONS_RESPONSEHEADERS:
      return "responseHeaders";
    case ON_RESPONSE_STARTED_OPTIONS_EXTRAHEADERS:
      return "extraHeaders";
    case ON_RESPONSE_STARTED_OPTIONS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnResponseStartedOptions ParseOnResponseStartedOptions(const std::string& enum_string) {
  if (enum_string == "responseHeaders")
    return ON_RESPONSE_STARTED_OPTIONS_RESPONSEHEADERS;
  if (enum_string == "extraHeaders")
    return ON_RESPONSE_STARTED_OPTIONS_EXTRAHEADERS;
  return ON_RESPONSE_STARTED_OPTIONS_NONE;
}


const char* ToString(OnBeforeRedirectOptions enum_param) {
  switch (enum_param) {
    case ON_BEFORE_REDIRECT_OPTIONS_RESPONSEHEADERS:
      return "responseHeaders";
    case ON_BEFORE_REDIRECT_OPTIONS_EXTRAHEADERS:
      return "extraHeaders";
    case ON_BEFORE_REDIRECT_OPTIONS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnBeforeRedirectOptions ParseOnBeforeRedirectOptions(const std::string& enum_string) {
  if (enum_string == "responseHeaders")
    return ON_BEFORE_REDIRECT_OPTIONS_RESPONSEHEADERS;
  if (enum_string == "extraHeaders")
    return ON_BEFORE_REDIRECT_OPTIONS_EXTRAHEADERS;
  return ON_BEFORE_REDIRECT_OPTIONS_NONE;
}


const char* ToString(OnCompletedOptions enum_param) {
  switch (enum_param) {
    case ON_COMPLETED_OPTIONS_RESPONSEHEADERS:
      return "responseHeaders";
    case ON_COMPLETED_OPTIONS_EXTRAHEADERS:
      return "extraHeaders";
    case ON_COMPLETED_OPTIONS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnCompletedOptions ParseOnCompletedOptions(const std::string& enum_string) {
  if (enum_string == "responseHeaders")
    return ON_COMPLETED_OPTIONS_RESPONSEHEADERS;
  if (enum_string == "extraHeaders")
    return ON_COMPLETED_OPTIONS_EXTRAHEADERS;
  return ON_COMPLETED_OPTIONS_NONE;
}


const char* ToString(OnErrorOccurredOptions enum_param) {
  switch (enum_param) {
    case ON_ERROR_OCCURRED_OPTIONS_EXTRAHEADERS:
      return "extraHeaders";
    case ON_ERROR_OCCURRED_OPTIONS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnErrorOccurredOptions ParseOnErrorOccurredOptions(const std::string& enum_string) {
  if (enum_string == "extraHeaders")
    return ON_ERROR_OCCURRED_OPTIONS_EXTRAHEADERS;
  return ON_ERROR_OCCURRED_OPTIONS_NONE;
}


RequestFilter::RequestFilter()
 {}

RequestFilter::~RequestFilter() = default;
RequestFilter::RequestFilter(RequestFilter&& rhs) = default;
RequestFilter& RequestFilter::operator=(RequestFilter&& rhs) = default;
// static
bool RequestFilter::Populate(
    const base::Value& value, RequestFilter* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  const base::Value* urls_value = dict.Find("urls");
  if (!urls_value) {
    return false;
  }
  {
    if (!(*urls_value).is_list()) {
      return false;
    }
    else {
      if (!json_schema_compiler::util::PopulateArrayFromList((*urls_value).GetList(), &out->urls)) {
        return false;
      }
    }
  }

  const base::Value* types_value = dict.Find("types");
  if (types_value) {
    {
      if (!(*types_value).is_list()) {
        return false;
      }
      else {
        out->types.emplace();
        for (const auto& it : ((*types_value)).GetList()) {
          ResourceType tmp;
          const std::string* resource_type_as_string = (it).GetIfString();
          if (!resource_type_as_string) {
            return false;
          }
          tmp = ParseResourceType(*resource_type_as_string);
          if (tmp == RESOURCE_TYPE_NONE) {
            return false;
          }
          out->types->push_back(tmp);
        }
      }
    }
  }

  const base::Value* tab_id_value = dict.Find("tabId");
  if (tab_id_value) {
    {
      auto temp = (*tab_id_value).GetIfInt();
      if (!temp.has_value()) {
        out->tab_id = absl::nullopt;
        return false;
      }
      out->tab_id = *temp;
    }
  }

  const base::Value* window_id_value = dict.Find("windowId");
  if (window_id_value) {
    {
      auto temp = (*window_id_value).GetIfInt();
      if (!temp.has_value()) {
        out->window_id = absl::nullopt;
        return false;
      }
      out->window_id = *temp;
    }
  }

  return true;
}

// static
std::unique_ptr<RequestFilter> RequestFilter::FromValue(const base::Value& value) {
  auto out = std::make_unique<RequestFilter>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict RequestFilter::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("urls", json_schema_compiler::util::CreateValueFromArray(this->urls));

  if (this->types) {
    {
      std::vector<std::string> types_list;
      for (const auto& it : *(this->types)) {
      types_list.push_back(web_request::ToString(it));
    }
    to_value_result.Set("types", json_schema_compiler::util::CreateValueFromArray(types_list));
    }

  }
  if (this->tab_id) {
    to_value_result.Set("tabId", *this->tab_id);

  }
  if (this->window_id) {
    to_value_result.Set("windowId", *this->window_id);

  }

  return to_value_result;
}


UploadData::UploadData()
 {}

UploadData::~UploadData() = default;
UploadData::UploadData(UploadData&& rhs) = default;
UploadData& UploadData::operator=(UploadData&& rhs) = default;
// static
bool UploadData::Populate(
    const base::Value& value, UploadData* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  const base::Value* bytes_value = dict.Find("bytes");
  if (bytes_value) {
    {
      out->bytes = (*bytes_value).Clone();
    }
  }

  const base::Value* file_value = dict.Find("file");
  if (file_value) {
    {
      auto* temp = (*file_value).GetIfString();
      if (!temp) {
        out->file = absl::nullopt;
        return false;
      }
      out->file = *temp;
    }
  }

  return true;
}

// static
std::unique_ptr<UploadData> UploadData::FromValue(const base::Value& value) {
  auto out = std::make_unique<UploadData>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict UploadData::ToValue() const {
  base::Value::Dict to_value_result;

  if (this->bytes) {
    to_value_result.Set("bytes", (this->bytes)->Clone());

  }
  if (this->file) {
    to_value_result.Set("file", *this->file);

  }

  return to_value_result;
}


FormDataItem::FormDataItem()
 {}

FormDataItem::~FormDataItem() = default;
FormDataItem::FormDataItem(FormDataItem&& rhs) = default;
FormDataItem& FormDataItem::operator=(FormDataItem&& rhs) = default;
// static
bool FormDataItem::Populate(
    const base::Value& value, FormDataItem* out) {
  if (value.type() == base::Value::Type::BINARY) {
    {
      if (!value.is_blob()) {
        return false;
      }
      else {
        out->as_binary = value.GetBlob();
      }
    }
    return true;
  }
  if (value.type() == base::Value::Type::STRING) {
    {
      auto* temp = value.GetIfString();
      if (!temp) {
        out->as_string = absl::nullopt;
        return false;
      }
      out->as_string = *temp;
    }
    return true;
  }
  return false;
}

// static
std::unique_ptr<FormDataItem> FormDataItem::FromValue(const base::Value& value) {
  auto out = std::make_unique<FormDataItem>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value FormDataItem::ToValue() const {
  base::Value result;
  if (as_binary) {
    DCHECK(result.is_none()) << "Cannot set multiple choices for form_data_item";
    result = base::Value(base::Value(*as_binary));

  }
  if (as_string) {
    DCHECK(result.is_none()) << "Cannot set multiple choices for form_data_item";
    result = base::Value(*as_string);

  }
  DCHECK(!result.is_none()) << "Must set at least one choice for form_data_item";
  return result;
}


const char* ToString(IgnoredActionType enum_param) {
  switch (enum_param) {
    case IGNORED_ACTION_TYPE_REDIRECT:
      return "redirect";
    case IGNORED_ACTION_TYPE_REQUEST_HEADERS:
      return "request_headers";
    case IGNORED_ACTION_TYPE_RESPONSE_HEADERS:
      return "response_headers";
    case IGNORED_ACTION_TYPE_AUTH_CREDENTIALS:
      return "auth_credentials";
    case IGNORED_ACTION_TYPE_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

IgnoredActionType ParseIgnoredActionType(const std::string& enum_string) {
  if (enum_string == "redirect")
    return IGNORED_ACTION_TYPE_REDIRECT;
  if (enum_string == "request_headers")
    return IGNORED_ACTION_TYPE_REQUEST_HEADERS;
  if (enum_string == "response_headers")
    return IGNORED_ACTION_TYPE_RESPONSE_HEADERS;
  if (enum_string == "auth_credentials")
    return IGNORED_ACTION_TYPE_AUTH_CREDENTIALS;
  return IGNORED_ACTION_TYPE_NONE;
}



//
// Functions
//

namespace HandlerBehaviorChanged {

base::Value::List Results::Create() {
  base::Value::List create_results;

  return create_results;
}
}  // namespace HandlerBehaviorChanged

//
// Events
//

namespace OnBeforeRequest {

const char kEventName[] = "webRequest.onBeforeRequest";

Details::RequestBody::FormData::FormData()
 {}

Details::RequestBody::FormData::~FormData() = default;
Details::RequestBody::FormData::FormData(FormData&& rhs) = default;
Details::RequestBody::FormData& Details::RequestBody::FormData::operator=(FormData&& rhs) = default;
base::Value::Dict Details::RequestBody::FormData::ToValue() const {
  base::Value::Dict to_value_result;

  for (const auto& it : additional_properties) {
    to_value_result.Set(it.first, json_schema_compiler::util::CreateValueFromArray(it.second));

  }

  return to_value_result;
}



Details::RequestBody::RequestBody()
 {}

Details::RequestBody::~RequestBody() = default;
Details::RequestBody::RequestBody(RequestBody&& rhs) = default;
Details::RequestBody& Details::RequestBody::operator=(RequestBody&& rhs) = default;
base::Value::Dict Details::RequestBody::ToValue() const {
  base::Value::Dict to_value_result;

  if (this->error) {
    to_value_result.Set("error", *this->error);

  }
  if (this->form_data) {
    to_value_result.Set("formData", (this->form_data)->ToValue());

  }
  if (this->raw) {
    to_value_result.Set("raw", json_schema_compiler::util::CreateValueFromArray(*this->raw));

  }

  return to_value_result;
}



Details::Details()
: frame_id(0),
parent_frame_id(0),
document_lifecycle(extension_types::DOCUMENT_LIFECYCLE_NONE),
frame_type(extension_types::FRAME_TYPE_NONE),
tab_id(0),
type(RESOURCE_TYPE_NONE),
time_stamp(0.0) {}

Details::~Details() = default;
Details::Details(Details&& rhs) = default;
Details& Details::operator=(Details&& rhs) = default;
base::Value::Dict Details::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("requestId", this->request_id);

  to_value_result.Set("url", this->url);

  to_value_result.Set("method", this->method);

  to_value_result.Set("frameId", this->frame_id);

  to_value_result.Set("parentFrameId", this->parent_frame_id);

  if (this->document_id) {
    to_value_result.Set("documentId", *this->document_id);

  }
  if (this->parent_document_id) {
    to_value_result.Set("parentDocumentId", *this->parent_document_id);

  }
  if (this->document_lifecycle != extension_types::DOCUMENT_LIFECYCLE_NONE) {
    to_value_result.Set("documentLifecycle", extension_types::ToString(this->document_lifecycle));

  }
  if (this->frame_type != extension_types::FRAME_TYPE_NONE) {
    to_value_result.Set("frameType", extension_types::ToString(this->frame_type));

  }
  if (this->request_body) {
    to_value_result.Set("requestBody", (this->request_body)->ToValue());

  }
  to_value_result.Set("tabId", this->tab_id);

  to_value_result.Set("type", web_request::ToString(this->type));

  if (this->initiator) {
    to_value_result.Set("initiator", *this->initiator);

  }
  to_value_result.Set("timeStamp", this->time_stamp);


  return to_value_result;
}


base::Value::List Create(const Details& details) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((details).ToValue());

  return create_results;
}

}  // namespace OnBeforeRequest

namespace OnErrorOccurred {

const char kEventName[] = "webRequest.onErrorOccurred";

Details::Details()
: frame_id(0),
parent_frame_id(0),
document_lifecycle(extension_types::DOCUMENT_LIFECYCLE_NONE),
frame_type(extension_types::FRAME_TYPE_NONE),
tab_id(0),
type(RESOURCE_TYPE_NONE),
time_stamp(0.0),
from_cache(false) {}

Details::~Details() = default;
Details::Details(Details&& rhs) = default;
Details& Details::operator=(Details&& rhs) = default;
base::Value::Dict Details::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("requestId", this->request_id);

  to_value_result.Set("url", this->url);

  to_value_result.Set("method", this->method);

  to_value_result.Set("frameId", this->frame_id);

  to_value_result.Set("parentFrameId", this->parent_frame_id);

  to_value_result.Set("documentId", this->document_id);

  if (this->parent_document_id) {
    to_value_result.Set("parentDocumentId", *this->parent_document_id);

  }
  to_value_result.Set("documentLifecycle", extension_types::ToString(this->document_lifecycle));

  to_value_result.Set("frameType", extension_types::ToString(this->frame_type));

  to_value_result.Set("tabId", this->tab_id);

  to_value_result.Set("type", web_request::ToString(this->type));

  if (this->initiator) {
    to_value_result.Set("initiator", *this->initiator);

  }
  to_value_result.Set("timeStamp", this->time_stamp);

  if (this->ip) {
    to_value_result.Set("ip", *this->ip);

  }
  to_value_result.Set("fromCache", this->from_cache);

  to_value_result.Set("error", this->error);


  return to_value_result;
}


base::Value::List Create(const Details& details) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((details).ToValue());

  return create_results;
}

}  // namespace OnErrorOccurred

namespace OnActionIgnored {

const char kEventName[] = "webRequest.onActionIgnored";

Details::Details()
: action(IGNORED_ACTION_TYPE_NONE) {}

Details::~Details() = default;
Details::Details(Details&& rhs) = default;
Details& Details::operator=(Details&& rhs) = default;
base::Value::Dict Details::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("requestId", this->request_id);

  to_value_result.Set("action", web_request::ToString(this->action));


  return to_value_result;
}


base::Value::List Create(const Details& details) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((details).ToValue());

  return create_results;
}

}  // namespace OnActionIgnored

}  // namespace web_request
}  // namespace api
}  // namespace extensions

